import logging

from digiformatter import styles

__all__ = ["debug", "info", "warn", "error", "critical", "log"]

styles.create("debug", fg="blue", prefix="DBG")                               # DEBUG
styles.create("info", prefix="INF")                                           # INFO
styles.create("warning", fg="yellow", prefix="WRN")                           # WARNING
styles.create("warn", fg="yellow", prefix="WRN")                              # WARN
styles.create("error", fg="red", attr="bold", prefix="ERR")                   # ERROR
styles.create("critical", fg="yellow", bg="red", attr="bold", prefix="CRT")   # CRITICAL
styles.create("fatal", fg="yellow", bg="red", attr="bold", prefix="CRT")      # FATAL


def debug(message, **kwargs):
    log(message, level="debug", **kwargs)


def info(message, **kwargs):
    log(message, level="info", **kwargs)


def warn(message, **kwargs):
    log(message, level="warning", **kwargs)


def error(message, **kwargs):
    log(message, level="error", **kwargs)


def critical(message, **kwargs):
    log(message, level="critical", **kwargs)


def log(message, level="info", showtime=None, showprefix=None):
    styles.print(message, style=level, showtime=showtime, showprefix=showprefix)


class DigiFormatterHandler(logging.Handler):
    def __init__(self, *args, showsource=False, **kwargs):
        self.showsource = showsource
        super().__init__(*args, **kwargs)

    def emit(self, record):
        message = record.getMessage()
        if self.showsource:
            message = f"{record.name}: {message}"
        log(message, level=record.levelname.lower())


# Get the next unassigned log level, higher than the base level specified (default to 20: INFO)
def getFreeLevel(base=None):
    baselevel = logging._nameToLevel.get(base, 20)
    level = baselevel
    while level in logging._levelToName:
        level += 1
    return level


def addLogLevel(name, *args, base=None, **kwargs):
    styles.create(name, *args, **kwargs)
    freelevel = getFreeLevel(base)
    logging.addLevelName(freelevel, name)
    return freelevel



import os
import math

__all__ = ["cursorUp", "cursorDown", "cursorRight", "cursorLeft", "scrollUp", "scrollDown", "setWindowTitle", "overwriteLines", "createLoadBar", "truncate"]

# Activate VT-100 terminal formatting
os.system("")

linelength = 100
windowheight = 40

# ASCII/ANSI characters
BLANK = " "
BELL = BEL = "\x07"

# VT-100 escape sequences
ESC = "\033"

CURSOR_UP = C_UP = ESC + "A"
CURSOR_DOWN = C_DOWN = ESC + "B"
CURSOR_RIGHT = C_RIGHT = ESC + "C"
CURSOR_LEFT = C_LEFT = ESC + "D"
CURSOR_HOME = C_HOME = ESC + "H"

SAVE_CURSOR = C_SAVE = ESC + "7"
LOAD_CURSOR = C_LOAD = ESC + "8"

START_CURSOR_BLINKING = C_BLINK_ON = ESC + "[?12h"
STOP_CURSOR_BLINKING = C_BLINK_OFF = ESC + "[?12l"
SHOW_CURSOR = C_SHOW = ESC + "[?25h"
HIDE_CURSOR = C_HIDE = ESC + "[?25l"

SCROLL_UP = ESC + "[1S"
SCROLL_DOWN = ESC + "[1S"

INSERT_BLANK = INS_BLANK = ESC + "[1@"
DELETE_CHAR = DEL_CHAR = ESC + "[1P"
ERASE_CHAR = ERS_CHAR = ESC + "[1X"
INSERT_LINE = INS_LINE = ESC + "[1L"
DELETE_LINE = DEL_CHAR = ESC + "[1M"

CLEAR_LINE_FROM_CURSOR_RIGHT = CLEAR_RIGHT = ESC + "[0K"
CLEAR_LINE_FROM_CURSOR_LEFT = CLEAR_LEFT = ESC + "[1K"

CLEAR_SCREEN_FROM_CURSOR_DOWN = CLEAR_DOWN = ESC + "[0J"
CLEAR_SCREEN_FROM_CURSOR_UP = CLEAR_UP = ESC + "[1J"

CLEAR_SCREEN = CLEAR = CLS = ESC + "[2J"

END_OF_LINE = EOL = CURSOR_RIGHT * linelength
BEGIN_OF_LINE = BOL = CURSOR_LEFT * linelength

RESET_TERMINAL = RESET = ESC + "c"


# Cursor movement functions
def cursorUp(amount):
    """Move the screen cursor to the up"""
    print(ESC + f"[{amount}A")


def cursorDown(amount):
    """Move the screen cursor to the down"""
    print(ESC + f"[{amount}B")


def cursorRight(amount):
    """Move the screen cursor to the right"""
    print(ESC + f"[{amount}C")


def cursorLeft(amount):
    """Move the screen cursor to the left"""
    print(ESC + f"[{amount}D")


def goto(x, y):
    "Go to an X,Y coordinate in the window"
    print(BEGIN_OF_LINE)
    cursorUp(windowheight)
    cursorRight(x)
    cursorDown(y)


def scrollUp(amount):
    """Scroll up"""
    print(ESC + f"[{amount}S")


def scrollDown(amount):
    """Scroll down"""
    print(ESC + f"[{amount}T")


def setWindowTitle(s):
    """Set window title"""
    print(ESC + f"2;{s}{BELL}")


def overwriteLines(lines):
    """Delete an amount of lines"""
    print(BEGIN_OF_LINE + ((CURSOR_UP + DELETE_LINE) * lines))


def createLoadBar(current, total, barlength = 50, showpercent = False):
    """Create a progress bar"""
    TWENTYFIVE = "\u2591"
    FIFTY = "\u2592"
    SEVENTYFIVE = "\u2593"
    FULL = "\u2588"
    shades = {
        0: " ",
        1: TWENTYFIVE,
        2: FIFTY,
        3: SEVENTYFIVE,
        4: FULL
    }

    complete = current / total          # fraction complete
    bartodraw = complete * barlength    # how long the bar should be

    fullbar = math.floor(bartodraw)     # how many full segments the bar should have
    fullbarstring = FULL * fullbar

    remainder = bartodraw - fullbar     # fraction remaining to draw
    shade = math.floor(remainder * 4)   # what shade the last segment should be
    remainderstring = shades[shade]

    barstring = fullbarstring + remainderstring
    if showpercent:
        percentage = round(complete * 100, 2)
        barstring = f"{barstring} {percentage}%"
    return barstring


def truncate(s, trunclen = 80, trailing = False):
    """Truncate a long string for terminal printing"""
    if len(s) > trunclen:
        if trailing:
            s = "…" + s[-(trunclen - 1):]
        else:
            s = s[(trunclen - 1):] + "…"
    return s



import logging

from digiformatter import logger as digilogger


def test_digiformathandler():
    logger = logging.getLogger("sizebot")
    dfhandler = digilogger.DigiFormatterHandler()
    logger.handlers = []
    logger.propogate = False
    logger.addHandler(dfhandler)
    logger.warning("AAAAA")



__version__ = "0.5.7.1"

__all__ = ["cursorUp", "cursorDown", "cursorRight", "cursorLeft", "scrollUp", "scrollDown", "setWindowTitle", "overwriteLines", "createLoadBar", "truncate", "goto", "styles"]

from digiformatter.formatter import cursorUp, cursorDown, cursorRight, cursorLeft, scrollUp, scrollDown, setWindowTitle, overwriteLines, createLoadBar, truncate, goto
from digiformatter.styles import styles



from time import strftime, localtime

import colored

__all__ = ["styles"]


class Style:
    __slots__ = ["name", "codes", "prefix", "showtime", "showprefix"]

    def __init__(self, name, codes, prefix = None, showtime = None, showprefix = None):
        self.name = name
        self.codes = codes
        self.prefix = prefix
        self.showtime = showtime
        self.showprefix = showprefix


class Styles:
    __slots__ = ["_styles", "default", "timestring", "timestampCodes"]

    def __init__(self):
        self._styles = {}
        self.timestring = "%d %b %H:%M:%S"
        self.timestampCodes = colored.fg("magenta")

    def create(self, name, *, fg = None, bg = None, attr = None, prefix = None, showtime = None, showprefix = None):
        """Create a custom style"""
        name = name.lower()
        codes = ""
        if fg is not None:
            codes += colored.fg(fg)
        if bg is not None:
            codes += colored.bg(bg)
        if attr is not None:
            codes += colored.attr(attr)
        self._styles[name] = Style(name, codes, prefix, showtime, showprefix)

    def format(self, message, *, style="default", showtime=None, showprefix=None):
        """Format a message in the requested style"""
        style = style.lower()
        if style not in self._styles:
            raise ValueError(f"Unknown style: {style}")
        styledata = self._styles[style]
        showtime = getFirstNotNone([showtime, styledata.showtime, True])
        showprefix = getFirstNotNone([showprefix, styledata.showprefix, True])
        formatted = ""
        if showtime:
            formatted += self._timestamp()
        if showprefix and styledata.prefix:
            formatted += styledata.codes + colored.attr("reverse") + styledata.prefix + colored.attr("reset") + " "
        for line in message.splitlines():
            formatted += styledata.codes + line + colored.attr("reset")
        return formatted

    def print(self, *args, **kwargs):
        """Print a message in the requested style"""
        print(self.format(*args, **kwargs))

    def _timestamp(self):
        """Color styling for terminal messages"""
        t = localtime()
        return self.timestampCodes + strftime(f"{self.timestring} | ", t) + colored.attr("reset")

    def __getattr__(self, name):
        name = name.lower()
        if name not in self._styles:
            raise AttributeError(f"{self.__class__.__name__!r} object has no attribute {name!r}")
        return lambda message: self.print(message, style=name)

    def __str__(self):
        return "styles: " + (" ".join(self.format(level, style=level, showtime = False, showprefix = False) for level in self._styles.keys()))


# Returns the first not None value in a list
def getFirstNotNone(items):
    for i in items:
        if i is not None:
            return i
    return None


styles = Styles()



import re
from pathlib import Path
import setuptools


def getRequirements():
    requirements = []
    with open("requirements.txt") as f:
        requirements = f.read().splitlines()
    return requirements


def getLongDescription():
    with open("README.md", "r", encoding="utf-8") as fh:
        longDescription = fh.read()
    return longDescription


def getVersion():
    path = Path(__file__).parent.resolve() / "digiformatter" / "__init__.py"
    with open(path, "r") as fp:
        version_file = fp.read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if not version_match:
        raise RuntimeError("Unable to find version string.")
    version = version_match.group(1)
    return version


setuptools.setup(
    name="digiformatter",
    version=getVersion(),
    author="DigiDuncan",
    author_email="digiduncan@gmail.com",
    description="Make your terminals look spicy.",
    long_description=getLongDescription(),
    long_description_content_type="text/markdown",
    url="https://github.com/DigiDuncan/DigiFormatter",
    python_requires=">=3",
    install_requires=getRequirements(),
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
    ]
)
